from baelfire.task.template import FirstTemplateTask


class SetupPy(FirstTemplateTask):
    source_name = 'template_setuppy'
    output_name = 'setuppy'

    def phase_settings(self):
        super().phase_settings()
        self.paths.set_path(
            'template_setuppy',
            'pyptemplates',
            'setuppy.jinja2',
        )
        self.paths.set_path('setuppy', 'cwd', 'setup.py')



from baelfire.dependencies import FileDoesNotExists
from baelfire.task import SubprocessTask


class BaseVirtualenv(SubprocessTask):

    def phase_settings(self):
        super().phase_settings()

        self.paths.set_path(
            'virtualenv:base',
            'cwd',
            'venv_%(settings:package_name)s',
        )
        self.paths.set_path('virtualenv:bin', 'virtualenv:base', 'bin')
        self.paths.set_path('exe:python', 'virtualenv:bin', 'python')
        self.paths.set_path('exe:pip', 'virtualenv:bin', 'pip')

    def python(self, command, *args, **kwargs):
        command = self.paths['exe:python'] + ' ' + command
        return self.popen([command], *args, **kwargs)

    def pip(self, command, *args, **kwargs):
        command = self.paths['exe:pip'] + ' ' + command
        return self.popen([command], *args, **kwargs)


class VirtualenvTask(BaseVirtualenv):

    def create_dependecies(self):
        self.add_dependency(FileDoesNotExists('virtualenv:base'))

    def build(self):
        cmd = 'virtualenv ' + self.paths['virtualenv:base']
        self.popen([cmd])



from baelfire.dependencies import FileChanged
from baelfire.dependencies import TaskDependency
from baelfire.task import Task

from .project import SetupPy
from .virtualenv import BaseVirtualenv
from .virtualenv import VirtualenvTask


class Create(Task):

    def create_dependecies(self):
        self.add_dependency(TaskDependency(VirtualenvTask()))
        self.add_dependency(TaskDependency(SetupPy()))

    def build(self):
        pass


class Develop(BaseVirtualenv):
    output_name = 'exe:python'

    @property
    def output(self):
        return self.paths[self.output_name]

    def create_dependecies(self):
        self.add_dependency(TaskDependency(Create()))
        self.add_dependency(FileChanged('setuppy'))

    def build(self):
        cmd = '%(setuppy)s develop' % self.paths
        self.python(cmd)
        self.popen(['touch %(exe:python)s' % self.paths])



from bael import project
from baelfire.core import Core
from baelfire.filedict import FileDict
from os.path import abspath
from os.path import dirname
from os.path import exists
from os.path import join


class ProjectCore(Core):

    def before_dependencies(self):
        super().before_dependencies()
        data = self.get_data_settings()

        self.paths['cwd'] = dirname(self.find_pyproject_file())
        self.paths['pyproject'] = dirname(project.__file__)
        self.paths.set_path('pyptemplates', 'pyproject', 'templates')
        self.settings['project_name'] = data['name']
        self.settings['package_name'] = data['package name']

        self.paths.set_path('package:src', 'cwd', 'src')
        self.paths.set_path('package:main', 'src', '%(settings:package_name)s')

    def get_data_settings(self):
        path = self.find_pyproject_file()
        data = FileDict(path)
        data.load(True)
        data.ensure_key_exists('name', 'Project name')
        data.ensure_key_exists('package name', 'Package name')
        data.save()
        return data

    def find_pyproject_file(self, path=None):
        if not path:
            path = abspath('./')
        file_path = join(path, '.pyproject.yaml')
        if exists(file_path):
            return file_path
        else:
            toppath = dirname(path)
            if toppath == path:
                return '.pyproject.yaml'
            return self.find_pyproject_file(toppath)



from .core import ProjectCore
from argparse import ArgumentParser
from baelfire.application.application import Application
from baelfire.application.commands.graph.graph import Graph
from logging import getLogger

log = getLogger(__name__)


class BaelApplication(Application):
    core_cls = ProjectCore

    tasks = {
        'develop': 'bael.project.develop:Develop',
        'create': 'bael.project.develop:Create',
    }

    def create_parser(self):
        self.parser = ArgumentParser()
        self._add_task_group()
        self._add_logging_group()

    def _add_task_group(self):
        tasks = self.parser.add_argument_group(
            'Tasks',
            'Project related options',
        )

        tasks.add_argument(
            '-c',
            '--create',
            dest='create',
            help='Create full project path.',
            action="store_true",
        )
        tasks.add_argument(
            '-d',
            '--develop',
            dest='develop',
            help='Download requiretments.',
            action="store_true",
        )
        tasks.add_argument(
            '-g',
            '--graph',
            dest='graph',
            help='Draw task dependency graph.',
            action="store_true",
        )

    def run_command_or_print_help(self, args):
        if args.create or args.develop:
            task = self._get_task(args)
            try:
                try:
                    task.run()
                finally:
                    report_path = task.save_report()
            except:
                log.error('Error in %s' % (report_path,))
                raise
            if args.graph:
                Graph(report_path).render()
        else:
            self.parser.print_help()

    def _get_task(self, args):
        if args.create:
            return self.import_task(self.tasks['create'])(self.core_cls())
        elif args.develop:
            return self.import_task(self.tasks['develop'])(self.core_cls())
        else:
            raise RuntimeError('No task selected!')


def run():
    BaelApplication().run()



from os import fwalk
from os import mkdir
from os import path

from baelfire.dependencies import FileDoesNotExists
from baelfire.dependencies import RunBefore
from baelfire.dependencies.file import FileDependency
from baelfire.task import Task


class MissingInitFiles(FileDependency):

    def should_build(self):
        try:
            for root, dirs, files, rootfd in fwalk(self.path):
                make = True
                for dirname in self.settings['ignore_inits_dirs']:
                    if dirname in root:
                        make = False
                        break
                if make and '__init__.py' not in files:
                    return True
        except FileNotFoundError:
            return True
        return False


class SrcFolderTask(Task):

    def create_dependecies(self):
        self.add_dependency(FileDoesNotExists('package:src'))

    def build(self):
        mkdir(self.paths['package:src'])


class MainFolderTask(Task):

    def create_dependecies(self):
        self.add_dependency(RunBefore(SrcFolderTask()))
        self.add_dependency(FileDoesNotExists('package:main'))

    def build(self):
        mkdir(self.paths['package:main'])


class FillWithInitsTask(Task):

    output_name = 'package:main'

    def create_dependecies(self):
        self.add_dependency(RunBefore(MainFolderTask()))
        self.add_dependency(MissingInitFiles(self.output_name))

    def phase_settings(self):
        super().phase_settings()
        self.settings['ignore_inits_dirs'] = [
            '__pycache__',
        ]

    def build(self):
        for root, dirs, files, rootfd in fwalk(self.paths[self.output_name]):
            make = True
            for dirname in self.settings['ignore_inits_dirs']:
                if dirname in root:
                    make = False
                    break
            if make and '__init__.py' not in files:
                open(path.join(root, '__init__.py'), 'w').close()



# -*- encoding: utf-8 -*-
from setuptools import setup, find_packages

install_requires = [
    'baelfire>=0.3.2',
    'virtualenv',
]

if __name__ == '__main__':
    setup(
        name='bael.project',
        version='0.2.4',
        description='Simple project generator.',
        license='Apache License 2.0',
        packages=find_packages('src'),
        package_dir={'': 'src'},
        namespace_packages=['bael'],
        install_requires=install_requires,
        zip_safe=False,
        package_data={
            '': ['project/templates/*.jinja2'],
        },
        entry_points="""\
              [console_scripts]
                  pyproject = bael.project.cmd:run
          """,
    )
